function y=AR1_Sigma(T, alpha, sigma)
% NAME            : AR1_Simple.m
% Original name   : AR1.m
% Description     : Generate simple Gaussian AR(1) without controlling the variance.
%                   I.e., it is a Gaussina AR with sigma^2=1. 
% Usage           : function y=AR1_Simple(T,a)
% Input           : T - Sample size
% Input           : alpha - Autoregressive coefficient
% Input           : sigma - standard deviation of AR(1) process
% Output          : y - Vector of AR1
% Note            : sigma is std. dev. NOT variance
%==========================================================================
% AUTHOR          : ALESSANDRO CASINI
% Email           : acasini@bu.edu  
% Date            : First version: 11/30/2015
%==========================================================================
% DISCLAIMER       
%==========================================================================  
% Please let me know if you find any bugs or encounter any problems while 
% using this code. All feedback is appreciated.
% @copyright, 2015

%==========================================================================
% DESCRIPTIONS 
%==========================================================================

%==========================================================================
% COMMENTS
%==========================================================================

%% ========================================================================
%             Preliminaries
%  ========================================================================

e = sigma*randn(T*2,1);
y(1,1)=sigma*randn;
%y(1,1)=0;
for i=2:T*2
    y(i,1)=alpha*y(i-1,1)+e(i,1);
end

y=y(T+1:end,1);