% NAME            : M1_Power.m
% Description     : Model M1
%==========================================================================
% AUTHOR          : ALESSANDRO CASINI
% Email           : alessandro.casini@uniroma2.it or acasini@bu.edu
% Date            : First version: 2018
% Last update     : 2/2/2021
%==========================================================================
% DISCLAIMER
%==========================================================================
% Please let me know if you find any bugs or encounter any problems while
% using this code. All feedback is appreciated.
% @copyright, 2021
%% ========================================================================
%                        Preliminaries
% =========================================================================
rng(1)
clear all; close all;  clc;

sims = 5000;
N_b = 50;
N_d = 11;
T = 500;

b = linspace(0.02,1,N_b);
M = floor(b*T);
delta = linspace(0,5,N_d); % for local power


%delta_t=0;
beta0 = 0;
sigma_e=1;
lambda0=0.2;
rho1=0.8;
rho2=0.1;
V = lambda0*1/(1-2*rho1+rho1^2)+(1-lambda0)*1/(1-2*rho2+rho2^2);
c = delta*sqrt(V);


%NW_Lag = ceil(4*(T/100)^(2/9));                                             % This is b1T^-1
%h2T=T^-0.20;

% M=bT
%% Critical Values
%load cv_fixedb_bartlett_inf_01
%cv_tstat_hac = [-1.96,1.96];
cv_tstat_hac = [-1.645,1.645]; % for one sided test, same as KV
%cv_NW_Fixed_b_KVB=[-4.771, 4.771];
%% Preallocation
rho = zeros(T+1,1);
%e = zeros(T+1,1);
Rej_NW = zeros(N_b,N_d,sims);
Rej_NW_fixedb = zeros(N_b,N_d,sims);
Rej_NW_fixedb_inf = zeros(N_b,N_d,sims);
Rej_NW_fixedb_feasible_rec = zeros(N_b,N_d,sims);
%% Initial conditions
% e(1)=randn;
%% ========================================================================
%                       CV Infeasible Nonstationary Fixed-b
%==========================================================================
T_cv = 1000;
r = (1:T_cv)/T_cv;
Tlambda = floor(T_cv*lambda0);
rho_cv(1,1:T_cv) = rho1;
rho_cv(Tlambda+1:T_cv) = rho2;
Sigma = 1./sqrt(1-2*rho_cv+rho_cv.^2);
%Sigma(1,1)=0.1;
%Sigma = 2;
N = 50000;
Nb = 50;
omega = zeros(Nb,N);
tstat = zeros(Nb,N);
b = linspace(0,1,Nb+1);
b(1) = [];
for j_cv = 1:N
    dW = 1/sqrt(T_cv)*randn(1,T_cv);
    tvdW = Sigma.*dW;
    tvW = cumsum(Sigma.*dW);
    tvB = tvW-r*tvW(T_cv);
    tvB_l = tvB;
    %tvB_l = [0,tvB(1:T-1)];
    %plot(tvbb)
    for i = 1:Nb
        shift = floor(b(i)*T_cv);
        if i < Nb
            omega(i,j_cv) =2/b(i)*sum(tvB_l.*tvB_l*1/T_cv)-1/b(i)*...
                sum((tvB_l(shift+1:T_cv).*tvB_l(1:T_cv-shift)+tvB_l(1:T_cv-shift).*tvB_l(shift+1:T_cv))*1/T_cv);
        else
            omega(i,j_cv) =2/b(i)*sum(tvB_l.*tvB_l*1/T_cv);
        end
        tstat(i,j_cv) = tvW(T_cv)/sqrt(omega(i,j_cv));
    end
end
tstat_sort = sort(tstat,2);
cv_ns_fixedb = quantile(tstat_sort,0.95,2);
%% ========================================================================
%                       DGP
%==========================================================================
for t=1:floor(lambda0*T)
    rho(t)=rho1;
end
for t=floor(lambda0*T)+1:T+1
    %rho(t)=0.7; KV (2005)
    rho(t) = rho2;
end
parfor j=1:sims
    disp(j)
    %noise
    u=randn(T+1,1);
    u(1) = 0;
    e = zeros(T+1,1);
    e(1)=randn;
    for t=2:floor(lambda0*T)
        e(t)= rho(t)*e(t-1) + u(t);
    end
    e(floor(lambda0*T))=randn;
    for t=floor(lambda0*T)+1:T+1
        e(t)= rho(t)*e(t-1) + u(t);
    end
    e=e(2:T+1);
    %figure
    %plot(e)
    %H0
    for ci = 1:N_d
        beta = beta0+c(ci)/sqrt(T);
        y=  beta + e;
        X=ones(T,1);
        [beta_hat,~,residual] = regress(y,X);
        zeta_hat = X.*residual;
        XX = (X'*X)/T;
        %% CV Feasible Nonstationary Fixed-b
        %Sigma_hat= tvnw(zeta_hat);
        %cv_fixedb_tv_feasible=tv_fixedb_feasible(Sigma_hat, 0.95);
        %% CV Feasible Nonstationary Fixed-b, rectangular kernel
        Sigma_hat_rectangular= tvnw_rectangular(zeta_hat);
        %Sigma_hat_rectangular(1:200)=5;
        %Sigma_hat_rectangular(201:end)=1.11;
        cv_fixedb_tv_feasible_rectangular=tv_fixedb_feasible(Sigma_hat_rectangular, 0.95);
        %% CV Feasible Nonstationary Fixed-b, symmetric kernel
        %Sigma_hat_symmetric= tvnw_symm_adj(zeta_hat);
        %cv_fixedb_tv_feasible_symm=tv_fixedb_feasible(Sigma_hat_symmetric, 0.95);
        for bi=1:N_b
            %% Newey-West (1987) - Predetermined Bandwidth
            HAC_NW = XX^-1 * nw(zeta_hat, M(bi)) * XX^-1;
            tNW = T^.5 *(beta_hat(1) -beta0) / HAC_NW(1,1)^.5;
            %if (cv_tstat_hac(1)< tNW && tNW < cv_tstat_hac(2)) % two sided
            if tNW < cv_tstat_hac(2) % one sides, same as SV
                Rej_NW(bi,ci,j) = 0;
            else
                Rej_NW(bi,ci,j) = 1;
            end
            %% Newey-West (1987) - KVB (stationary) Fixed-b crtitical value (we need to take the cv from KVB)
            %if -cv_fixedb_bartlett(0.975,b)< tNW && tNW < cv_fixedb_bartlett(0.975,b) % two sided
            if tNW < cv_fixedb_bartlett(0.95,b(bi)) % one sides, same as SV
                Rej_NW_fixedb(bi,ci,j) = 0;
            else
                Rej_NW_fixedb(bi,ci,j) = 1;
            end
            %% Newey-West (1987) - Nonsta Fixed-b crtitical value (we need to simulate our distribution and get the cv)
            %if tNW < tstat_sort(bi,0.95*50000) % one sides, same as SV
            if tNW < cv_ns_fixedb(bi)
                Rej_NW_fixedb_inf(bi,ci,j) = 0;
            else
                Rej_NW_fixedb_inf(bi,ci,j) = 1;
            end
            %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (This needs to estimate \Sigma(u) e get the critical values
            %if tNW < cv_fixedb_tv_feasible(bi) % one sides, same as KV
            %    Rej_NW_fixedb_feasible(j,bi) = 0;
            %else
            %    Rej_NW_fixedb_feasible(j,bi) = 1;
            %end
            %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (rectangular kernel)
            if tNW < cv_fixedb_tv_feasible_rectangular(bi) % one sides, same as KV
                Rej_NW_fixedb_feasible_rec(bi,ci,j) = 0;
            else
                Rej_NW_fixedb_feasible_rec(bi,ci,j) = 1;
            end
            %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (symmetric kernel)
            %if tNW < cv_fixedb_tv_feasible_symm(bi) % one sides, same as KV
            %    Rej_NW_fixedb_feasible_symm(j,bi) = 0;
            %else
            %    Rej_NW_fixedb_feasible_symm(j,bi) = 1;
            %end
        end
    end
end

Rej_rates_NW = mean(Rej_NW,3);
%Size_NW_pre = 1-Rej_rates_NW_pre; %???
Rej_rates_NW_fixedb = mean(Rej_NW_fixedb,3);
% Size_NW_Fixedb_KVB(1,bi) = 1-Rej_rates_NW_Fixedb_KVB(1,bi);
% Rej_rates_NW_Fixedb_Nonsta(1,bi) = mean(Rej_NW_Fixedb_Nonsta(:, bi));
% Size_NW_Fixedb_Nonsta(1,bi) = 1-Rej_rates_NW_Fixedb_Nonsta(1,bi);
Rej_rates_NW_fixedb_inf = mean(Rej_NW_fixedb_inf,3);
%Rej_rates_NW_fixedb_fea = mean(Rej_NW_fixedb_feasible);
Rej_rates_NW_fixedb_fea_rec = mean(Rej_NW_fixedb_feasible_rec,3);
%Rej_rates_NW_fixedb_fea_symm = mean(Rej_NW_fixedb_feasible_symm);
%% Do figure with size as in KV (2005, ET)
% for each method
%b_index = [1,5,15,25,35,45,50]; % b = 0.02, 0.1, 0.3, 0.5, 0.7, 0.9, 1, KV
b_index = [15,25,35,50]; % b = 0.3, 0.5, 0.7, 1
figure
subplot(2,2,1)
hold on
plot(delta,Rej_rates_NW(b_index(1),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW(b_index(2),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW(b_index(3),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW(b_index(4),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$b=0.3$', '$b=0.5$','$b=0.7$','$b=1.0$','FontSize', 18, 'Interpreter','latex','Location','northwest')
title('$N(0,1)$','FontSize', 18,'Interpreter','latex')
subplot(2,2,2)
hold on
plot(delta,Rej_rates_NW_fixedb(b_index(1),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(2),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(3),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(4),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$b=0.3$', '$b=0.5$','$b=0.7$','$b=1.0$','FontSize', 18, 'Interpreter','latex','Location','northwest')
title('Stationary fixed-$b$ (KV)','FontSize', 18,'Interpreter','latex')
subplot(2,2,3)
hold on
plot(delta,Rej_rates_NW_fixedb_inf(b_index(1),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(2),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(3),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(4),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$b=0.3$', '$b=0.5$','$b=0.7$','$b=1.0$','FontSize', 18, 'Interpreter','latex','Location','northwest')
title('Infeasible nonstat fixed-$b$','FontSize', 18,'Interpreter','latex')
subplot(2,2,4)
hold on
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(1),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(2),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(3),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(4),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$b=0.3$', '$b=0.5$','$b=0.7$','$b=1.0$','FontSize', 18, 'Interpreter','latex','Location','northwest')
title('Feasible nonstat fixed-$b$','FontSize', 18,'Interpreter','latex')

% for each b
figure
subplot(2,2,1)
hold on
plot(delta,Rej_rates_NW(b_index(1),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(1),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(1),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(1),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$N(0,1)$','Stationary fixed-$b$ (KV)','Infeasible nonstat fixed-$b$','feasible nonstat fixed-$b$',...
    'FontSize', 12,'Interpreter','latex','Location','northwest')
title('$b = 0.3$','FontSize', 18,'Interpreter','latex')
subplot(2,2,2)
hold on
plot(delta,Rej_rates_NW(b_index(2),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(2),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(2),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(2),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$N(0,1)$','Stationary fixed-$b$ (KV)','Infeasible nonstat fixed-$b$','feasible nonstat fixed-$b$',...
    'FontSize', 12,'Interpreter','latex','Location','northwest')
title('$b = 0.5$','FontSize', 18,'Interpreter','latex')
subplot(2,2,3)
hold on
plot(delta,Rej_rates_NW(b_index(3),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(3),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(3),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(3),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$N(0,1)$','Stationary fixed-$b$ (KV)','Infeasible nonstat fixed-$b$','feasible nonstat fixed-$b$',...
    'FontSize', 12,'Interpreter','latex','Location','northwest')
title('$b = 0.7$','FontSize', 18,'Interpreter','latex')
subplot(2,2,4)
hold on
plot(delta,Rej_rates_NW(b_index(4),:),':','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb(b_index(4),:),'--','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_inf(b_index(4),:),'-.','linewidth',1.5,'color',[0.07,0.62,1.00])
plot(delta,Rej_rates_NW_fixedb_fea_rec(b_index(4),:),'-','linewidth',1.5,'color',[0.07,0.62,1.00])
ylim([0,1])
xlabel('$\delta$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Power','FontSize', 12,'Interpreter','latex')
legend('$N(0,1)$','Stationary fixed-$b$ (KV)','Infeasible nonstat fixed-$b$','feasible nonstat fixed-$b$',...
    'FontSize', 12,'Interpreter','latex','Location','northwest')
title('$b = 1.0$','FontSize', 18,'Interpreter','latex')
