% NAME            : M1_Power.m
% Description     : Model M1
%==========================================================================
% AUTHOR          : ALESSANDRO CASINI
% Email           : alessandro.casini@uniroma2.it or acasini@bu.edu
% Date            : First version: 2018
% Last update     : 2/2/2021
%==========================================================================
% DISCLAIMER
%==========================================================================
% Please let me know if you find any bugs or encounter any problems while
% using this code. All feedback is appreciated.
% @copyright, 2021
%% ========================================================================
%                        Preliminaries
rng(1)
close all; clear; clc;
sims = 5000;
delta_t=0;
beta0 = 0;
sigma_e=1;
lambda0=0.1;
T=2000;
NW_Lag = ceil(4*(T/100)^(2/9));                                             % This is b1T^-1
h2T=T^-0.20;
N_b = 50;
b = linspace(0.02,1,N_b);
M = floor(b*T);
% M=bT
%% Critical Values
load cv_fixedb_bartlett_inf_01
%cv_tstat_hac = [-1.96,1.96];
cv_tstat_hac = [-1.645,1.645]; % for one sided test, same as KV
%cv_NW_Fixed_b_KVB=[-4.771, 4.771];
%% Preallocation
rho = zeros(T+1,1);
e = zeros(T+1,1);
Rej_NW = zeros(sims,N_b);
Rej_NW_fixedb = zeros(sims,N_b);
Rej_NW_fixedb_inf = zeros(sims,N_b);
%% Initial conditions
e(1)=randn;

%% ========================================================================
%                       DGP
%==========================================================================

for t=1:floor(lambda0*T)
    rho(t)=0.95;
end
for t=floor(lambda0*T)+1:T+1
    %rho(t)=0.7; KV (2005)
    rho(t) = 0.3;
end
for j=1:sims
    %noise
    u=randn(T+1,1);
    u(1) = 0;
    for t=2:T+1
        e(t)= rho(t)*e(t-1) + u(t);
    end
    e=e(2:T+1);
    %H0
    y=  beta0 + e;
    X=ones(T,1);
    [beta_hat,~,residual] = regress(y,X);
    zeta_hat = X.*residual;
    XX = (X'*X)/T;
    for bi=1:N_b
        %% Newey-West (1987) - Predetermined Bandwidth
        HAC_NW = XX^-1 * nw(zeta_hat, M(bi)) * XX^-1;
        tNW = T^.5 *(beta_hat(1) -beta0) / HAC_NW(1,1)^.5;
        %if (cv_tstat_hac(1)< tNW && tNW < cv_tstat_hac(2)) % two sided
        if tNW < cv_tstat_hac(2) % one sides, same as SV
            Rej_NW(j, bi) = 0;
        else
            Rej_NW(j, bi) = 1;
        end
        %% Newey-West (1987) - KVB (stationary) Fixed-b crtitical value (we need to take the cv from KVB)
        %if -cv_fixedb_bartlett(0.975,b)< tNW && tNW < cv_fixedb_bartlett(0.975,b) % two sided
        if tNW < cv_fixedb_bartlett(0.95,b(bi)) % one sides, same as SV
            Rej_NW_fixedb(j,bi) = 0;
        else
            Rej_NW_fixedb(j, bi) = 1;
        end
            %% Newey-West (1987) - Nonsta Fixed-b crtitical value (we need to simulate our distribution and get the cv)
        if tNW < tstat_sort(bi,0.95*50000) % one sides, same as SV
            Rej_NW_fixedb_inf(j,bi) = 0;
        else
            Rej_NW_fixedb_inf(j,bi) = 1;
        end
    end
    

    %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (This needs to estimate \Sigma(u) e get the critical values
end

Rej_rates_NW = mean(Rej_NW);
%Size_NW_pre = 1-Rej_rates_NW_pre; %???
Rej_rates_NW_fixedb = mean(Rej_NW_fixedb);
% Size_NW_Fixedb_KVB(1,bi) = 1-Rej_rates_NW_Fixedb_KVB(1,bi);
% Rej_rates_NW_Fixedb_Nonsta(1,bi) = mean(Rej_NW_Fixedb_Nonsta(:, bi));
% Size_NW_Fixedb_Nonsta(1,bi) = 1-Rej_rates_NW_Fixedb_Nonsta(1,bi);
Rej_rates_NW_fixedb_inf = mean(Rej_NW_fixedb_inf);
%% Do figure with size as in KV (2005, ET)
figure
hold on
plot(b,Rej_rates_NW)
plot(b,Rej_rates_NW_fixedb)
plot(b,Rej_rates_NW_fixedb_inf)
legend('HAC','fixedb','fixedb_inf','Location','northwest')
ylim([0,0.35])

