% NAME            : M1_Power.m
% Description     : Model M1
%==========================================================================
% AUTHOR          : ALESSANDRO CASINI
% Email           : alessandro.casini@uniroma2.it or acasini@bu.edu
% Date            : First version: 2018
% Last update     : 2/2/2021
%==========================================================================
% DISCLAIMER
%==========================================================================
% Please let me know if you find any bugs or encounter any problems while
% using this code. All feedback is appreciated.
% @copyright, 2021
%% ========================================================================
%                        Preliminaries
% =========================================================================
rng(1)
clear all; close all;  clc;
sims = 5000;
delta_t=0;
beta0 = 0;
T=500;
NW_Lag = ceil(4*(T/100)^(2/9));                                             % This is b1T^-1
h2T=T^-0.20;
N_b = 50;
b = linspace(0.02,1,N_b);
M = floor(b*T);
% M=bT
%% Critical Values
%load cv_fixedb_bartlett_inf_01
%cv_tstat_hac = [-1.96,1.96];
cv_tstat_hac = [-1.645,1.645]; % for one sided test, same as KV
%cv_NW_Fixed_b_KVB=[-4.771, 4.771];
%% Preallocation
rho = zeros(T+1,1);
e = zeros(T+1,1);
Rej_NW = zeros(sims,N_b);
Rej_NW_fixedb = zeros(sims,N_b);
Rej_NW_fixedb_inf = zeros(sims,N_b);
%% Initial conditions
%e(1)=randn;
%% ========================================================================
%                       CV Infeasible Nonstationary Fixed-b
%==========================================================================
T_cv = 1000;
r = (1:T_cv)/T_cv;
for t_cv=1:T_cv
    rho_cv(t_cv)=0.85*(cos(1.5*t_cv/T_cv));
end
sigma_u=sqrt(1.5);
Sigma = sigma_u./sqrt(1-2*rho_cv+rho_cv.^2);
%Sigma(1,1)=0.1;
%Sigma = 2;
N = 50000;
Nb = 50;
omega = zeros(Nb,N);
tstat = zeros(Nb,N);
b = linspace(0,1,Nb+1);
b(1) = [];
for j_cv = 1:N
    dW = 1/sqrt(T_cv)*randn(1,T_cv);
    tvdW = Sigma.*dW;
    tvW = cumsum(Sigma.*dW);
    tvB = tvW-r*tvW(T_cv);
    tvB_l = tvB;
    %tvB_l = [0,tvB(1:T-1)];
    %plot(tvbb)
    for i = 1:Nb
        shift = floor(b(i)*T_cv);
        if i < Nb
            omega(i,j_cv) =2/b(i)*sum(tvB_l.*tvB_l*1/T_cv)-1/b(i)*...
                sum((tvB_l(shift+1:T_cv).*tvB_l(1:T_cv-shift)+tvB_l(1:T_cv-shift).*tvB_l(shift+1:T_cv))*1/T_cv);
        else
            omega(i,j_cv) =2/b(i)*sum(tvB_l.*tvB_l*1/T_cv);
        end
        tstat(i,j_cv) = tvW(T_cv)/sqrt(omega(i,j_cv));
    end
end
tstat_sort = sort(tstat,2);
cv_ns_fixedb = quantile(tstat_sort,0.95,2);
%% ========================================================================
%                       DGP
%==========================================================================
for j=1:sims
    e= TVAR1_M2(T, sigma_u, 0);
    y=  beta0 + e;
    X=ones(T,1);
    [beta_hat,~,residual] = regress(y,X);
    zeta_hat = X.*residual;
    XX = (X'*X)/T;
    %% CV Feasible Nonstationary Fixed-b 
    %Sigma_hat= tvnw(zeta_hat);
    %cv_fixedb_tv_feasible=tv_fixedb_feasible(Sigma_hat, 0.95);
    %% CV Feasible Nonstationary Fixed-b, rectangular kernel 
    Sigma_hat_rectangular= tvnw_rectangular(zeta_hat);
    %Sigma_hat_rectangular(1:200)=5;
    %Sigma_hat_rectangular(201:end)=1.11;
    cv_fixedb_tv_feasible_rectangular=tv_fixedb_feasible(Sigma_hat_rectangular, 0.95);
    %% CV Feasible Nonstationary Fixed-b, symmetric kernel 
    %Sigma_hat_symmetric= tvnw_symm_adj(zeta_hat);
    %cv_fixedb_tv_feasible_symm=tv_fixedb_feasible(Sigma_hat_symmetric, 0.95);
    for bi=1:N_b
        %% Newey-West (1987) - Predetermined Bandwidth
        HAC_NW = XX^-1 * nw(zeta_hat, M(bi)) * XX^-1;
        tNW = T^.5 *(beta_hat(1) -beta0) / HAC_NW(1,1)^.5;
        %if (cv_tstat_hac(1)< tNW && tNW < cv_tstat_hac(2)) % two sided
        if tNW < cv_tstat_hac(2) % one sides, same as SV
            Rej_NW(j, bi) = 0;
        else
            Rej_NW(j, bi) = 1;
        end
        %% Newey-West (1987) - KVB (stationary) Fixed-b crtitical value (we need to take the cv from KVB)
        %if -cv_fixedb_bartlett(0.975,b)< tNW && tNW < cv_fixedb_bartlett(0.975,b) % two sided
        if tNW < cv_fixedb_bartlett(0.95,b(bi)) % one sides, same as SV
            Rej_NW_fixedb(j,bi) = 0;
        else
            Rej_NW_fixedb(j, bi) = 1;
        end
        %% Newey-West (1987) - Nonsta Fixed-b crtitical value (we need to simulate our distribution and get the cv)
        %if tNW < tstat_sort(bi,0.95*50000) % one sides, same as SV
        if tNW < cv_ns_fixedb(bi)
            Rej_NW_fixedb_inf(j,bi) = 0;
        else
            Rej_NW_fixedb_inf(j,bi) = 1;
        end
        %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (This needs to estimate \Sigma(u) e get the critical values
        %if tNW < cv_fixedb_tv_feasible(bi) % one sides, same as KV
        %    Rej_NW_fixedb_feasible(j,bi) = 0;
        %else
        %    Rej_NW_fixedb_feasible(j,bi) = 1;
        %end
        %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (rectangular kernel)
        if tNW < cv_fixedb_tv_feasible_rectangular(bi) % one sides, same as KV
            Rej_NW_fixedb_feasible_rec(j,bi) = 0;
        else
            Rej_NW_fixedb_feasible_rec(j,bi) = 1;
        end
        %% Newey-West (1987) - Feasible Nonsta Fixed-b crtitical value (symmetric kernel)
        %if tNW < cv_fixedb_tv_feasible_symm(bi) % one sides, same as KV
        %    Rej_NW_fixedb_feasible_symm(j,bi) = 0;
        %else
        %    Rej_NW_fixedb_feasible_symm(j,bi) = 1;
        %end    
    end
end

Rej_rates_NW = mean(Rej_NW);
%Size_NW_pre = 1-Rej_rates_NW_pre; %???
Rej_rates_NW_fixedb = mean(Rej_NW_fixedb);
% Size_NW_Fixedb_KVB(1,bi) = 1-Rej_rates_NW_Fixedb_KVB(1,bi);
% Rej_rates_NW_Fixedb_Nonsta(1,bi) = mean(Rej_NW_Fixedb_Nonsta(:, bi));
% Size_NW_Fixedb_Nonsta(1,bi) = 1-Rej_rates_NW_Fixedb_Nonsta(1,bi);
Rej_rates_NW_fixedb_inf = mean(Rej_NW_fixedb_inf);
%Rej_rates_NW_fixedb_fea = mean(Rej_NW_fixedb_feasible);
Rej_rates_NW_fixedb_fea_rec = mean(Rej_NW_fixedb_feasible_rec);
%Rej_rates_NW_fixedb_fea_symm = mean(Rej_NW_fixedb_feasible_symm);
%% Do figure with size as in KV (2005, ET)
alphas=0.05*ones(1,51);
figure
hold on
plot(b,Rej_rates_NW, ':','linewidth',3,'color',[0.07,0.62,1.00])
plot(b,Rej_rates_NW_fixedb, '--','linewidth',3,'color',[0.07,0.62,1.00])
plot(b,Rej_rates_NW_fixedb_inf, '-.','linewidth',3,'color',[0.07,0.62,1.00])
plot(b,Rej_rates_NW_fixedb_fea_rec, '-','linewidth',3,'color',[0.07,0.62,1.00])
%plot([0,b], alphas,'-','linewidth',0.1,'color', 'black', 'HandleVisibility','off')
plot([0,b], alphas,'-','linewidth',0.1,'color', 'black')
%plot(b,Rej_rates_NW_fixedb_fea_symm)
xlabel('$b$', 'FontSize', 18, 'Interpreter','latex')
ylabel('Rejection Probability','FontSize', 18,'Interpreter','latex')
legend('$N(0,1)$','Stationary fixed-$b$ (KV)','Infeasible nonstat fixed-$b$', 'Feasible nonstat fixed-$b$', '0.05 level', 'Location','northwest', 'FontSize', 18, 'Interpreter','latex')
title('Null Rejection Probabilities for Model M2, $T=500$','FontSize', 20,'Interpreter','latex')
ylim([0,1])
%yline(0.05)


