%% NAME            : M_DM_Power.m
% Description      : Model M DM
%==========================================================================
% AUTHOR          : ALESSANDRO CASINI
% Email           : alessandro.casini@uniroma2.it or acasini@bu.edu  
% Date            : First version: 2018
% Last update     : 2/2/2021
%==========================================================================
% DISCLAIMER       
%==========================================================================  
% Please let me know if you find any bugs or encounter any problems while 
% using this code. All feedback is appreciated.
% @copyright, 2021
%% ========================================================================
%                        Preliminaries     
%==========================================================================
close all; clear all; clc;
sims = 1000;
AlphaAR_e = 0.3;
AlphaAR_X = 0.;
miu0=0;
delta=10;
miuX=1;
beta0 = 1;
theta1=-0.0;
sigma_e=1;
sigma_X=1;
T=400;
N0=0.4;
TT = T;
dof=[8,16,32,64];% Degrees of freedom
dof_index=1;
v=dof(1);
T_out = floor((1-N0)*T);
NW_Lag = ceil(4*(T_out/100)^(2/9));
%NW_Lag = 20;
nu_Cos = floor(0.41*T^(2/3));
Cos_Mat = cosine_weight(T,nu_Cos);
NW_Lag_Fixed_b = 1.5*T_out/v;
S1= floor(T_out^(4/25));
S1_Bartlett=floor(T_out^(4/15));
nT = floor(T_out^0.68);
        % nT  nT_Tes
nT_Test = floor(T_out^0.8);
nT_Bandw = floor(T_out^0.60);
nblock=floor(T_out/nT);
b2T=T_out^-0.20;
Tm = floor(T*N0);
tau=1;
lambda0=0.8;
tTest=1;
param=1;
%% Critical Values
Cri_NW_tstat = [-1.96,1.96];
NWcrits=[-2.4918, 2.4924;-2.2200,2.2135;-2.0896,2.0797;-2.0219,2.0233]'; %% Externally calculated, see readme
NWmat1= NWcrits(1,1);
NWmat2= NWcrits(2,1);
Cri_NW_Fixed_b = [NWmat1, NWmat2];
Cri_Andrews_Asy = [-1.96, 1.96];
Cri_val_J_HAC = [-1.96, 1.96];
cv_NW_Fixed_b_KVB=[-4.771, 4.771];
cv_EWP_tv = tinv(0.975,nu_Cos);
%% ========================================================================
%                       DGP     
%==========================================================================
for ii=1:length(TT)
    T=TT(ii);
 for j=1:sims
     X0=AR1_Miu_Sigma(T+tau, AlphaAR_X, miuX, sigma_X);

     % H1 and Break
     %X1=AR1_Miu_Sigma(T+tau, AlphaAR_X, miuX, sigma_X);
     X2=AR1_Miu_Sigma(T+tau, AlphaAR_X, miuX, sigma_X);
     X1=X0 + 1*randn;
     X2 = X0+0.2*X2+ 2*randn;
     X2_temp =  X2+delta;% 0+1*randn;
     X1_temp=X0;
     
     X2(T-180:end-160) = X2_temp(T-180:end-160);
 
     e=AR1_Sigma(T+tau, AlphaAR_e, sigma_e);
     e2=e;
     
     X11=ones(T+tau,1);
     X=[X11, X0];
     XX1=[X11, X1];
     XX2=[X11, X2];
     nreg=size(X,2);
     for t=(1+tau):(T+tau)
            y(t,1) = miu0 + beta0*X0(t-1,1) +  e2(t,1);
     end
    
     yy=y(2:end);
     yin = yy(1:Tm);
     Xin = XX1(1:Tm,:);
     results = ols_Results( yin, Xin); 
     betahat1 = results.beta;
     Xin = XX2(1:Tm,:);
     results = ols_Results( yin, Xin); 
     betahat2 = results.beta;
     Losses_1 = [];
     Losses_2 = [];  

     for it=1:(T-tau-Tm +1)                                                 % rows(m) is T (sample size)
        t= Tm + it - 1; 
     yhat1 = XX1(t,:)*betahat1;
     yhat2 = XX2(t,:)*betahat2;
     Ytrue = yy(t,1);
     Losses_1 = [Losses_1;  (Ytrue -  yhat1).^2];
     Losses_2 = [Losses_2;  (Ytrue -  yhat2).^2];  
     end
     d=Losses_1-Losses_2;
      Break_Dates=[];
     %figure()
     %plot(d)

     DM_NW_Pred=DMtest_NW_Pred(Losses_1, Losses_2, NW_Lag);
     if (Cri_NW_tstat(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_tstat(2))
        Rej_NW_pre(j) = 0;
     else
        Rej_NW_pre(j) = 1;
     end
     
     DM_test=DMtest_J_HAC_QS(Losses_1, Losses_2, S1, nblock, nT, b2T, Break_Dates);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_QS(j) = 0;
     else
        Rej_J_HAC_QS(j) = 1;
     end
     
     DM_NW_Pred=DMtest_NW_Auto(Losses_1, Losses_2, NW_Lag);
     if (Cri_NW_tstat(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_tstat(2))
        Rej_NW_Auto(j) = 0;
     else
        Rej_NW_Auto(j) = 1;
     end
     DM_NW_Pred=DMtest_NW_Auto_Prewhite(Losses_1, Losses_2, NW_Lag);
     if (Cri_NW_tstat(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_tstat(2))
        Rej_NW_Auto_Prewhite(j) = 0;
     else
        Rej_NW_Auto_Prewhite(j) = 1;
     end
     DM_NW_Pred=DMtest_NW_fixedb(Losses_1, Losses_2, NW_Lag_Fixed_b);
     if (Cri_NW_Fixed_b(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_Fixed_b(2))
        Rej_NW_fixedb(j) = 0;
     else
        Rej_NW_fixedb(j) = 1;
     end
     DM_NW_Pred=DMtest_NW_fixedb_KVB(Losses_1, Losses_2, T_out-1);
     if (cv_NW_Fixed_b_KVB(1)< DM_NW_Pred && DM_NW_Pred < cv_NW_Fixed_b_KVB(2))
        Rej_NW_fixedb_KVB(j) = 0;
     else
        Rej_NW_fixedb_KVB(j) = 1;
     end
     %% Newey-West (1987) - Fixed-b crtitical value - LLSW Rule
     DM_NW_Pred=DMtest_NW_fixedb_LLSW(Losses_1, Losses_2, NW_Lag_Fixed_b);
     if (Cri_NW_Fixed_b(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_Fixed_b(2))
        Rej_NW_Fixedb_LLSW(j) = 0;
     else
        Rej_NW_Fixedb_LLSW(j) = 1;
     end
     DM_NW_Pred=DMtest_Andrews91(Losses_1, Losses_2, NW_Lag);
     if (Cri_NW_tstat(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_tstat(2))
        Rej_Andrews91(j) = 0;
     else
        Rej_Andrews91(j) = 1;
     end
     DM_NW_Pred=DMtest_Andrews91_Prewhite(Losses_1, Losses_2, NW_Lag);
     if (Cri_NW_tstat(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_tstat(2))
        Rej_Andrews91_Prewhite(j) = 0;
     else
        Rej_Andrews91_Prewhite(j) = 1;
     end
     DM_NW_Pred=DMtest_EWP(Losses_1, Losses_2);
     if (Cri_NW_tstat(1)< DM_NW_Pred && DM_NW_Pred < Cri_NW_tstat(2))
        Rej_EWP(j) = 0;
     else
        Rej_EWP(j) = 1;
     end
     
     %% EWP_tv 
     DM_NW_Pred=DMtest_EWC(Losses_1, Losses_2, Cos_Mat,nu_Cos);
     if (-cv_EWP_tv< DM_NW_Pred && DM_NW_Pred < cv_EWP_tv)
        Rej_EWC(j) = 0;
     else
        Rej_EWC(j) = 1;
     end
     %% J HAC QS Automatic
     DM_test=DMtest_J_HAC_QS_Automatic(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_QS_Automatic(j) = 0;
     else
        Rej_J_HAC_QS_Automatic(j) = 1;
     end
     DM_test=DMtest_J_HAC_Bartlett_Automatic(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_Barteltt_Automatic(j) = 0;
     else
        Rej_J_HAC_Barteltt_Automatic(j) = 1;
     end
     DM_test=DMtest_J_HAC_Bartlett_Automatic_Prewhite(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_Barteltt_Automatic_Prewhite(j) = 0;
     else
        Rej_J_HAC_Barteltt_Automatic_Prewhite(j) = 1;
     end
     DM_test=DMtest_J_HAC_QS_Automatic_Prewhite(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_QS_Automatic_Prewhite(j) = 0;
     else
        Rej_J_HAC_QS_Automatic_Prewhite(j) = 1;
     end
     DM_test=DMtest_J_HAC_QS_Automatic_Prewhite_SLS(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_QS_Automatic_Prewhite_SLS(j) = 0;
     else
        Rej_J_HAC_QS_Automatic_Prewhite_SLS(j) = 1;
     end
     DM_test=DMtest_J_HAC_Bartlett_Automatic_Prewhite_SLS(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_Barteltt_Automatic_Prewhite_SLS(j) = 0;
     else
        Rej_J_HAC_Barteltt_Automatic_Prewhite_SLS(j) = 1;
     end
     DM_test=DMtest_J_HAC_QS_Automatic_Prewhite_SLS_miu(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_QS_Automatic_Prewhite_SLS_miu(j) = 0;
     else
        Rej_J_HAC_QS_Automatic_Prewhite_SLS_miu(j) = 1;
     end
     DM_test=DMtest_J_HAC_Bartlett_Automatic_Prewhite_SLS_miu(Losses_1, Losses_2, S1, nT, nblock, b2T, Break_Dates, nT_Bandw);
     if (Cri_NW_tstat(1)< DM_test && DM_test < Cri_NW_tstat(2))
        Rej_J_HAC_Barteltt_Automatic_Prewhite_SLS_miu(j) = 0;
     else
        Rej_J_HAC_Barteltt_Automatic_Prewhite_SLS_miu(j) = 1;
     end
    
 end
 
end
 Rej_rates_NW_pre = mean(Rej_NW_pre);
 Size_NW_pre = 1-Rej_rates_NW_pre;
 Rej_rates_J_HAC_QS = mean(Rej_J_HAC_QS);
 Size_NW_pre = 1-Rej_rates_J_HAC_QS;
 Rej_rates_NW_Auto = mean(Rej_NW_Auto);
 Size_NW_Auto = 1-Rej_rates_NW_Auto;
 Rej_rates_NW_Auto_Prewhite = mean(Rej_NW_Auto_Prewhite);
 Size_NW_Auto_Prewhite = 1-Rej_rates_NW_Auto_Prewhite;
 Rej_rates_Andrews91 = mean(Rej_Andrews91);
 Size_Andrews91 = 1-Rej_rates_Andrews91;
 Rej_rates_Andrews91_Prewhite = mean(Rej_Andrews91_Prewhite);
 Size_Andrews91_Prewhite = 1-Rej_rates_Andrews91_Prewhite;
 Rej_rates_EWP = mean(Rej_EWP);
 Size_EWP = 1-Rej_rates_EWP;
 
 Rej_rates_J_HAC_QS_Automatic = mean(Rej_J_HAC_QS_Automatic);
 Size_NW_J_HAC_QS_Automatic = 1-Rej_rates_J_HAC_QS_Automatic;
 Rej_rates_J_HAC_Barteltt_Automatic = mean(Rej_J_HAC_Barteltt_Automatic);
 Size_NW_J_HAC_Barteltt_Automatic = 1-Rej_rates_J_HAC_Barteltt_Automatic;
 Rej_rates_J_HAC_Barteltt_Automatic_Prewhite = mean(Rej_J_HAC_Barteltt_Automatic_Prewhite);
 Size_NW_J_HAC_Barteltt_Automatic_Prewhite = 1-Rej_rates_J_HAC_Barteltt_Automatic_Prewhite;
 Rej_rates_J_HAC_QS_Automatic_Prewhite = mean(Rej_J_HAC_QS_Automatic_Prewhite);
 Size_NW_J_HAC_QS_Automatic_Prewhite = 1-Rej_rates_J_HAC_QS_Automatic_Prewhite;
 Rej_rates_J_HAC_QS_Automatic_Prewhite_SLS = mean(Rej_J_HAC_QS_Automatic_Prewhite_SLS);
 Size_NW_J_HAC_QS_Automatic_Prewhite_SLS = 1-Rej_rates_J_HAC_QS_Automatic_Prewhite_SLS;
 Rej_rates_J_HAC_Barteltt_Automatic_Prewhite_SLS = mean(Rej_J_HAC_Barteltt_Automatic_Prewhite_SLS);
 Size_NW_J_HAC_Barteltt_Automatic_Prewhite_SLS = 1-Rej_rates_J_HAC_Barteltt_Automatic_Prewhite_SLS;
 Rej_rates_J_HAC_QS_Automatic_Prewhite_SLS_miu = mean(Rej_J_HAC_QS_Automatic_Prewhite_SLS_miu);
 Size_NW_J_HAC_QS_Automatic_Prewhite_SLS_miu = 1-Rej_rates_J_HAC_QS_Automatic_Prewhite_SLS_miu;
 Rej_rates_J_HAC_Barteltt_Automatic_Prewhite_SLS_miu = mean(Rej_J_HAC_Barteltt_Automatic_Prewhite_SLS_miu);
 Size_NW_J_HAC_Barteltt_Automatic_Prewhite_SLS_miu = 1-Rej_rates_J_HAC_Barteltt_Automatic_Prewhite_SLS_miu;
 Rej_rates_NW_fixedb = mean(Rej_NW_fixedb);
 Size_NW_pre = 1-Rej_rates_NW_fixedb;
 Rej_rates_NW_fixedb_KVB = mean(Rej_NW_fixedb_KVB);
 Size_NW_fixedb_KVB = 1-Rej_rates_NW_fixedb_KVB;
 Rej_rates_NW_fixedb_LLSW = mean(Rej_NW_Fixedb_LLSW);
 Size_NW_fixedb_LLSW = 1-Rej_rates_NW_fixedb_LLSW;
 Rej_rates_EWC = mean(Rej_EWC);
 Size_EWC = 1-Rej_rates_EWC;
 
 save('M_DM_200_10');